@@ -5,19 +5,17 @@
 #include "misc.h"
 
 
-extern Qps 	*qps;
-extern Svec<Command> commands;
-extern ControlBar *control_bar;
+extern List<Command*> commands;
+extern QStringList	hidden_process;
+extern ControlBar	*control_bar;
 extern int	default_font_height;
 extern bool 	flag_show_thread;	
 extern bool 	previous_flag_show_thread;
 extern int 	flag_thread_ok;
 extern int 	num_opened_files;
 
-extern SearchBox *search_box2;
-extern SearchBox *search_box;
-//TrayIcon *trayicon=NULL;
-//QPushButton *command_button[16]; //testing
-extern XButton *xb; //testing
+extern Qps			*qps;
+extern SearchBox 	*search_box;
+extern TFrame 		*infobox;
 
 #endif	// GLOBAL_H
