@@ -63,7 +63,6 @@ enum fields {
 	F_AFFCPU,
 	F_SLPAVG,
 #endif
-
 	F_NLWP,
 #ifdef SOLARIS
 	F_ARCH,
@@ -71,8 +70,11 @@ enum fields {
 	F_MAJFLT, F_MINFLT,
 #ifdef LINUX
 	F_TRS, F_DRS,
+	F_STACK,
 #endif
-	F_SIZE, F_SWAP, F_RSS,
+	F_SIZE,
+	F_SWAP,  // Linux not correct 
+	F_RSS,
 #ifdef LINUX
 	F_SHARE, F_DT,
 #endif
@@ -168,28 +170,36 @@ class NameValue
 		const char *name;
 		const char *value;
 };
+#define CPU_TIMES(cpu, kind) cpu_times_vec[cpu * CPUTIMES + kind]
+
 
 // Process Infomation CLASS
 class Procinfo
 {
 	public:
 		Procinfo(int pid);
+		Procinfo(int pid, int thread);  //solaris !
 		~Procinfo();
 		Procinfo *ref() { refcnt++; return this; };
 		void deref() { if(!--refcnt)  delete this; };
 
-		int readproc(int pid);
+//		int readproc();		// new
+		int readproc(int pid=-1);
+		int readproc(int pid, int thread);
+		inline void calculate_cpu();
+
 		static void init_static();
 		static void read_common();
 		static void read_loadavg();
 		static int read_file(char *name, void *buf, int max);
+
 		void read_fd(int fdnum, char *path);
 		bool read_fds();
 		bool read_maps();
+		bool read_environ();
 #ifdef SOLARIS
 		void read_pmap_maps();
 #endif
-		bool read_environ();
 #ifdef MOSIX
 		static void check_for_mosix();
 		static Svec<int> mosix_nodes();
@@ -201,19 +211,27 @@ class Procinfo
 		static void read_usockets();
 		static void invalidate_sockets();
 #endif
+
 		int get_policy();
 		int get_rtprio();
 #ifdef LINUX
 		double get_tms();
 		unsigned long get_affcpu();
 #endif
+		bool first_run; 		// for optimized code 
 		int pid;
+        int pid_prev;
 
-		QString command;		//COMMAND
+		QString command;	//COMMAND
 		QString cmdline; 	//COMMAND_LINE
-	
-		int test_stop;  // for test
-		int type; 	// X,NETWORK,FILE_OPEN,TERMINAL(tty),THREAD,
+		QString username; 	//
+		QString groupname; 	//
+		QString cwd;		// null if not read
+		QString root;		// null if not read
+
+        bool accepted;      
+		int test_stop;  	// for test
+		int type; 		    // X,NETWORK,FILE_OPEN,TERMINAL(tty),THREAD,
 		
 		int uid, euid;
 		int gid, egid;
@@ -221,14 +239,14 @@ class Procinfo
 		char state;
 		int ppid;
 		int pgrp;
-		int session;
+		int session;        //???
 		dev_t tty;
 
 #ifdef LINUX
-		int tgid;	// thread id
+		int tgid;		// thread id
  	   	double tms;
 		int slpavg;
-    		unsigned long affcpu;
+   		unsigned long affcpu;
 
 		int suid, fsuid;
 		int sgid, fsgid;		
@@ -243,21 +261,23 @@ class Procinfo
 		unsigned long cmajflt;
 #endif
 		long utime;
+		long old_utime;		// initial value = -1 ;
 		long cutime;
 		int priority;
 		int nice;
-		unsigned long starttime;	// in jiffies since boot
+		unsigned long starttime;// in jiffies since boot
 		unsigned long wchan;
 
-	// =========== Memory ====================
-		unsigned long size;	// SIZE: total memory (K)
+		// Memory 
+		unsigned long size;		// SIZE: total memory (K)
 		unsigned long resident;	// RSS: pages in resident set (non-swapped) (K) 
 #ifdef LINUX
 		unsigned long share;	// shared memory pages (mmaped) (K)
 		unsigned long trs;      // text resident set size (K)
-		unsigned long lrs;	// shared-lib resident set size (K)
-		unsigned long drs;	// data resident set size (K)
-		unsigned long dt;	// dirty pages (number of pages, not K)
+		unsigned long lrs;		// shared-lib resident set size (K)
+		unsigned long drs;		// data resident set size (K)
+		unsigned long dt;		// dirty pages (number of pages, not K), obsolute in Kernel 2.6
+		unsigned long stack;	// stack size (K)
 #endif
 
 		int nthreads;		// number of LWPs or tasks (in Linux words)
@@ -265,25 +285,25 @@ class Procinfo
 		int addr_bits;		// address bits (32 or 64)
 #endif
 
-		struct timeval tv;		// time when the snapshot was taken
+		struct timeval tv;	// time when the snapshot was taken
+		struct timeval old_tv;	// old 
 
 		// Posix.1b scheduling
-		int policy;			// -1 = uninitialized
+		int policy;		// -1 = uninitialized
 #ifdef SOLARIS
 		char policy_name[2];	// two first letters of scheduling class
 #endif
 		int rtprio;		// 0-99, higher can pre-empt lower (-1 = uninitialized)
 
-#ifdef LINUX
-		// from proc/XX/cpu (in 2.2 SMP kernels)
-		unsigned long *per_cpu_times; // vector of num_cpus times
-#endif
+
 		// Linux: the cpu used most of the time of the process
 		// Solaris: the cpu on which the process last ran
 		int which_cpu;
+		int old_which_cpu;
 
 		// computed %cpu and %mem since last update
 		float wcpu;		// what ? -> wheight cpu 
+		float old_wcpu;	// what ? -> wheight cpu 
 		float pcpu;		// percent cpu after last update
 		float pmem;
 
@@ -312,6 +332,11 @@ class Procinfo
 		static unsigned *cpu_times_vec;
 		static unsigned *old_cpu_times_vec;
 
+#ifdef LINUX
+		// from proc/XX/cpu (in 2.2 SMP kernels)
+		unsigned long *per_cpu_times; // vector of num_cpus times
+		unsigned long *old_per_cpu_times; // vector of num_cpus times
+#endif
 		static unsigned num_cpus;		// current number of CPUs
 		static unsigned old_num_cpus; 		// previous number of CPUs
 		static long num_process; 		//  number of process
@@ -337,14 +362,11 @@ class Procinfo
 		Svec<Fileinfo*> *fd_files;	// file names or NULL if not read
 		Svec<Mapsinfo*> *maps;	// maps or NULL if not read
 
-		QString cwd;		// null if not read
-		QString root;		// null if not read
-
 #ifdef LINUX
-		Svec<SockInode> *sock_inodes; // socket inodes or NULL if not read
+		Svec<SockInode*> *sock_inodes; // socket inodes or NULL if not read
 #endif
 
-		Svec<NameValue> *environ;	// environment or NULL if not read
+		Svec<NameValue*> environ;	// environment or NULL if not read
 		char *envblock;		// malloc()ed environment data block
 
 #ifdef SOLARIS
@@ -353,7 +375,7 @@ class Procinfo
 		static kstat_ctl_t *kc;		// NULL if kstat not opened
 #endif
 
-		Details *details;		// details window or NULL (backlink)
+		Details *detail;		// details window or NULL (backlink)
 
 		bool selected:1;	// true if selected in current view
 		bool generation:1;	// whether new or old generation (alternating)
@@ -386,20 +408,21 @@ class Category
 {
 	public:
 		Category(const char *heading, const char *explain)
-			: name(heading), help(explain) {};
+			: name(heading), help(explain),flag_int_value(false) {};
 		virtual ~Category();
 
 		virtual int alignment() = 0;
-		virtual QString string(Procinfo *p) = 0;  // ???
+		virtual QString string(Procinfo *p) = 0;  
 		virtual int width() = 0;
 		virtual int compare(Procinfo *a, Procinfo *b);
-		virtual int gap() { return 0; };
 
 		const char *name;
 		const char *help;
 		int index;
+		bool flag_int_value;	// for total sum , cat_memory , cat_int
 };
 
+// COMMON 
 class Cat_int : public Category
 {
 	public:
@@ -415,6 +438,22 @@ class Cat_int : public Category
 		int field_width;
 };
 
+// COMMON  for memory usage  
+class Cat_memory : public Category
+{
+	public:
+		Cat_memory(const char *heading, const char *explain,int w, unsigned long Procinfo::*member);
+		virtual int alignment() { return QTNAME(AlignRight); };
+		virtual QString string(Procinfo *p);
+		virtual int width() { return field_width; };
+		virtual int compare(Procinfo *a, Procinfo *b);
+
+	protected:
+		unsigned long Procinfo::*uintl_member;
+		int field_width;
+};
+
+
 class Cat_uintl : public Category
 {
 	public:
@@ -555,7 +594,6 @@ public:
 };
 #endif
 
-
 class Cat_time : public Category
 {
 	public:
@@ -604,7 +642,7 @@ class Proc
 		Proc();
 		void refresh();
 		void newproc(Procinfo *p);
-		int readTaskByPID(int pid);
+		int  read_pid_tasks(int pid);
 
 		Category *cat_by_name(const char *s);
 		int  field_id_by_name(const char *s);
@@ -637,6 +675,7 @@ class Procview
 		void deduce_fields();
 
 		Svec<Procinfo *> procs;
+		Svec<Procinfo *> tags_kernel;
 		Svec<Procinfo *> old_procs;
 		
 		// root_procs contains processes without parent; normally only init, but
@@ -651,12 +690,15 @@ class Procview
 		bool reversed;		// true if sorted backwards
 		bool treeview;		// true if viewed in tree form
 
-		enum procstates {ALL, OWNED, NROOT, RUNNING} viewproc;
-		enum fieldstates {USER = RUNNING + 1, JOBS, MEM, SCHED,CUSTOM} viewfields;
+		enum procstates {ALL, OWNED, NROOT, RUNNING, HIDDEN} ;
+		enum fieldstates {USER = HIDDEN + 1, JOBS, MEM, SCHED,CUSTOM};
+		int viewproc;
+		int viewfields;
 
 		// lists of fields to be used for different views, terminated by -1:
-		static int custom_fields[18];
+		static int custom_fields[32];
 		static int user_fields[];
+		static int user_fields_smp[];
 		static int jobs_fields[];
 		static int mem_fields[];
 #ifdef LINUX
