@@ -6,25 +6,25 @@
 #ifndef PSTABLE_H
 #define PSTABLE_H
 
+//#include "qps.h"
 #include "htable.h"
 #include "proc.h"
-
+//class Qps;
 class Pstable : public HeadedTable
 {
     Q_OBJECT
 public:
+//	Qps		*qps;
     Pstable(QWidget *parent);
 
-    void recompute_table_widths();
-    void repaint_changed();
     void transfer_selection();
     void set_sortcol();
     void setProcview(Procview *pv);
-    void setRows();
-    void set_initial_mode(bool treemode);
     void set_mode(bool treemode);
     virtual void moveCol(int col, int place);
     void update_customfield();
+    void refresh();
+	
 
 public slots:
     void selection_update(const Svec<int> *row);
@@ -46,13 +46,12 @@ protected:
     virtual NodeState folded(int row);
     virtual int parentRow(int row);
     virtual bool lastChild(int row);
+	virtual char* total_selectedRow(int col);
+
+//virtual void drawCellContents(int row, int col, int w, int h, QPainter *p);
 
-    virtual void drawCellContents(int row, int col, int w, int h, QPainter *p);
-	
 private:
-    Procview *procview;
-    int leftmostchanged;	// leftmost (physical) col whose width changed,
-				// or -1
+    	Procview *procview;
 };
 
 #endif // PSTABLE_H
