@@ -8,7 +8,6 @@
 
 #include <qwidget.h>
 #include <qmenubar.h>
-#include <qlist.h>
 #include <qcolor.h>
 #include <qtoolbutton.h>
 
@@ -23,32 +22,28 @@
 #include "command.h"
 
 #if QT_VERSION < 300
-#error Qt library version 3.0 or higher is needed for this version of qps
+#error Qt library version 3.1 or higher is needed for this version of qps
 #endif
 
 
-#define DEBUG(str) 
+//#define DEBUG(str) 
 #define FUNC_START
 #define FUNC_END
 
-//#define DEBUG(str) 	printf("DEBUG:%s\n",str)
-//#define FUNC_START	printf("DEBUG:%s START\n",__PRETTY_FUNCTION__);
-//#define FUNC_END	printf("DEBUG:%s END\n",__PRETTY_FUNCTION__);
-
-
 
 class Qps : public QWidget
 {
 	Q_OBJECT
 	public:
-		Qps(QWidget *parent = 0);
+		Qps();
 		~Qps();
 
 		void set_update_period(int milliseconds);
 		void setWindowGroup(QWidget *w);
 		void setCommand(int argc, char **argv);
-		void iconicStartFix();
 		void setIconSize(int w,int h) { icon_width=w,icon_height=h;}
+		void write_settings();
+		void save_settings();
 
 		QPixmap *get_load_icon();
 
@@ -77,12 +72,17 @@ class Qps : public QWidget
 		static bool normalize_nice;		// whether nice should be normalized (normal nice is 0 etc)
 		static bool use_pmap;		// use /proc/bin/pmap for map names
 #endif
-		static bool tree_table;		// ** Tree/Linear  **
+		static bool tree_table;
 		static bool tree_gadgets;	// use open/close gadgets (triangles)
 		static bool tree_lines;		// draw tree branch lines
 
 		
 		static bool comm_is_magic;	//DEL auto-remove COMM when switching
+		static bool flag_systray;		// trayicon , sytemtray
+		static bool flag_exit;
+		static bool flag_show;		// last qps-main_window state 
+		static QString font_name;	
+		static int  font_size;
 
 
 		// colors which may be set by the user
@@ -115,14 +115,14 @@ class Qps : public QWidget
 			MENU_SIGTERM, MENU_SIGHUP, MENU_SIGINT, MENU_SIGKILL, MENU_OTHERS,
 			MENU_PROCS, MENU_FIELDS, MENU_CUSTOM,
 			MENU_PATH, MENU_INFOBAR, MENU_CTRLBAR, MENU_CUMUL, MENU_PREFS,
+			MENU_ADD_HIDDEN,MENU_REMOVE_HIDDEN,
 #ifdef MOSIX
 			POPUP_MIGRATE,
 #endif
 			MENU_FIRST_COMMAND	// must be last
 		};
 
-		public slots:
-			// menu:
+	public slots:
 		void clicked_trayicon();
 		void sig_term();
 		void sig_hup();
@@ -145,51 +145,45 @@ class Qps : public QWidget
 		void menu_children();
 		void menu_dynasty();
 		void menu_custom();
-		void mig_menu(int id);	// MOSIX only
+		void menu_remove_field();
+		void menu_add_hiddenlist();
+		void menu_remove_hiddenlist();
 		void menu_edit_cmd();
+		void mig_menu(int id);	// MOSIX only
 		void make_command_menu();
 		void run_command(int id);
 		void view_menu(int id);
 		void save_quit();
 		void add_fields_menu(int id);
-		void menu_remove_field();
 
 		void context_row_menu(QPoint p);
 		void context_heading_menu(QPoint p, int col);
 
-		void update_selection_status();
+		void update_menu_selection_status();
 		void forced_update();
 		void field_added(int index);
 		void field_removed(int index);
 		void set_table_mode(bool treemode);
-		void col_reorder(int, int);
 
 		void details_closed(Details *d);
 		void open_details(int row);
 
-		void showMinimized ();	// when clicked "iconfy button"
-		void iconify_window(); //DEL 
 		void config_change();
-
-		void update_field();
+		
+		//virtual	void showMinimized();
 
 	protected:
 		// reimplementation of QWidget methods
-		void resizeEvent(QResizeEvent *);
 		void timerEvent(QTimerEvent *);
 		void closeEvent(QCloseEvent *);
 		bool event(QEvent *e);
-
-		void myHideEvent();
-		void myShowEvent();
-
+		
 		void timer_refresh();
 		void refresh_details();
 
 		void update_menu_status();
 		void transfer_selection();
 		void update_table(int update_col);
-		void update_menu_selection_status();
 		void adjust_popup_menu(QPopupMenu *m, bool cont);
 		bool all_selected_stopped();
 
@@ -203,10 +197,8 @@ class Qps : public QWidget
 		void sendsig(Procinfo *p, int sig);
 		void earlier_refresh();
 		bool read_settings();
-		void write_settings();
 	
 		bool load_settings();
-		void save_settings();
 		
 		void bar_visibility();
 		void set_default_icon();
@@ -252,7 +244,7 @@ class Qps : public QWidget
 		QPixmap *default_icon;
 		bool default_icon_set;	// true if default icon is current icon
 
-		int context_col;		// where heading context menu was clicked
+		int context_col;	// where heading context menu was clicked
 		static const char *color_name[NUM_COLORS];
 };
 
