@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qttableview.cpp,v 1.9 2006/03/07 16:25:34 fasthyun Exp $
+** $Id: qttableview.cpp,v 1.21 2006/08/29 21:03:21 fasthyun Exp $
 **
 ** Implementation of QtTableView class
 **
@@ -13,10 +13,11 @@
 **********************************************************************/
 
 #include "qttableview.h"
-#ifndef QT_NO_QTTABLEVIEW
-#include "qscrollbar.h"
-#include "qpainter.h"
-#include "qdrawutil.h"
+
+#include <qapplication.h> 
+#include <qscrollbar.h>
+#include <qpainter.h>
+#include <qdrawutil.h>
 #include <limits.h>
 
 enum ScrollBarDirtyFlags {
@@ -28,19 +29,18 @@ enum ScrollBarDirtyFlags {
     horSteps	  = 0x20,
     horRange	  = 0x40,
     horValue	  = 0x80,
-    verMask	  = 0x0F,
-    horMask	  = 0xF0
+    verMask		= 0x0F,
+    horMask		= 0xF0
 };
 
 
 #define HSBEXT horizontalScrollBar()->sizeHint().height()
 #define VSBEXT verticalScrollBar()->sizeHint().width()
 
-
 class QCornerSquare : public QWidget		// internal class
 {
-public:
-    QCornerSquare( QWidget *, const char* = 0 );
+	public:
+	QCornerSquare( QWidget *, const char* = 0 );
     void paintEvent( QPaintEvent * );
 };
 
@@ -146,22 +146,27 @@ void QCornerSquare::paintEvent( QPaintEvent * )
 QtTableView::QtTableView( QWidget *parent, const char *name, WFlags f )
     : QFrame( parent, name, f )
 {
-    nRows		 = nCols      = 0;	// zero rows/cols
-    xCellOffs		 = yCellOffs  = 0;	// zero offset
-    xCellDelta		 = yCellDelta = 0;	// zero cell offset
-    xOffs		 = yOffs      = 0;	// zero total pixel offset
-    cellH		 = cellW      = 0;	// user defined cell size
-    tFlags		 = 0;
-    vScrollBar		 = hScrollBar = 0;	// no scroll bars
-    cornerSquare	 = 0;
-    sbDirty		 = 0;
-    eraseInPaint	 = FALSE;
-    verSliding		 = FALSE;
-    verSnappingOff	 = FALSE;
-    horSliding		 = FALSE;
-    horSnappingOff	 = FALSE;
+    nRows			= nCols      = 0;	// zero rows/cols
+    xCellOffs		= yCellOffs  = 0;	// zero offset
+    xCellDelta		= yCellDelta = 0;	// zero cell offset
+    xOffs			= yOffs      = 0;	// zero total pixel offset
+    cellH			= cellW      = 0;	// user defined cell size
+    vScrollBar		= hScrollBar = 0;	// no scroll bars
+	yOffs_prev		= 0;
+    tFlags			= 0;
+    cornerSquare	= 0;
+    sbDirty			= 0;
+    eraseInPaint	= FALSE;
+    verSliding		= FALSE;
+    verSnappingOff	= FALSE;
+    horSliding		= FALSE;
+    horSnappingOff	= FALSE;
+    inSbUpdate		= FALSE;
+	flag_update		= false ; // added 
     coveringCornerSquare = FALSE;
-    inSbUpdate		 = FALSE;
+	
+	setWFlags (Qt::WNoAutoErase);
+    setBackgroundMode(  Qt::NoBackground); //flicker free
 }
 
 /*
@@ -175,18 +180,14 @@ QtTableView::~QtTableView()
     delete cornerSquare;
 }
 
-
-/*
-  \internal
-  Reimplements QWidget::setBackgroundColor() for binary compatibility.
-  \sa setPalette()
-*/
-
 void QtTableView::setBackgroundColor( const QColor &c )
 {
     QWidget::setBackgroundColor( c );
+  	backColor=c;
 }
 
+
+
 /*\reimp
 */
 
@@ -232,18 +233,18 @@ void QtTableView::show()
 void QtTableView::repaint( int x, int y, int w, int h, bool erase )
 {
     if ( !isVisible() || testWState(WState_BlockUpdates) )
-	return;
+        return;
     if ( w < 0 )
-	w = width()  - x;
+        w = width()  - x;
     if ( h < 0 )
-	h = height() - y;
+        h = height() - y;
     QRect r( x, y, w, h );
     if ( r.isEmpty() )
-	return; // nothing to do
+        return; // nothing to do
     QPaintEvent e( r );
 
     if ( erase && backgroundMode() != NoBackground )
-	eraseInPaint = TRUE;			// erase when painting
+		;//eraseInPaint=TRUE; // erase when painting
     paintEvent( &e );
     eraseInPaint = FALSE;
 }
@@ -316,10 +317,10 @@ void QtTableView::setNumCols( int cols )
 {
     if ( cols < 0 ) {
 #if defined(QT_CHECK_RANGE)
-	qWarning( "QtTableView::setNumCols: (%s) Negative argument %d.",
+	    qWarning( "QtTableView::setNumCols: (%s) Negative argument %d.",
 		 name( "unnamed" ), cols );
 #endif
-	return;
+	    return;
     }
     if ( nCols == cols )
 	return;
@@ -456,7 +457,7 @@ void QtTableView::setXOffset( int x )
 
 void QtTableView::setYOffset( int y )
 {
-    setOffset( xOffset(), y );
+    setOffset( xOffset(), y ,true );
 }
 
 /*
@@ -472,72 +473,76 @@ void QtTableView::setYOffset( int y )
 
 void QtTableView::setOffset( int x, int y, bool updateScrBars )
 {
-    if ( (!testTableFlags(Tbl_snapToHGrid) || xCellDelta == 0) &&
-	 (!testTableFlags(Tbl_snapToVGrid) || yCellDelta == 0) &&
-	 (x == xOffs && y == yOffs) )
-	return;
+	if ( (!testTableFlags(Tbl_snapToHGrid) || xCellDelta == 0) &&
+			(!testTableFlags(Tbl_snapToVGrid) || yCellDelta == 0) &&
+			(x == xOffs && y == yOffs) )
+		return;
 
-    if ( x < 0 )
-	x = 0;
-    if ( y < 0 )
-	y = 0;
+	if ( x < 0 )
+		x = 0;
+	if ( y < 0 )
+		y = 0;
 
-    if ( cellW ) {
-	if ( x > maxXOffset() )
-	    x = maxXOffset();
-	xCellOffs = x / cellW;
-	if ( !testTableFlags(Tbl_snapToHGrid) ) {
-	    xCellDelta	= (short)(x % cellW);
+	if ( cellW ) {
+		if ( x > maxXOffset() )
+			x = maxXOffset();
+		xCellOffs = x / cellW;
+		if ( !testTableFlags(Tbl_snapToHGrid) ) {
+			xCellDelta	= (short)(x % cellW);
+		} else {
+			x		= xCellOffs*cellW;
+			xCellDelta	= 0;
+		}
 	} else {
-	    x		= xCellOffs*cellW;
-	    xCellDelta	= 0;
+		int xn=0, xcd=0, col = 0;
+		while ( col < nCols-1 && x >= xn+(xcd=cellWidth(col)) ) {
+			xn += xcd;
+			col++;
+		}
+		xCellOffs = col;
+		if ( testTableFlags(Tbl_snapToHGrid) ) {
+			xCellDelta = 0;
+			x = xn;
+		} else {
+			xCellDelta = (short)(x-xn);
+		}
 	}
-    } else {
-	int xn=0, xcd=0, col = 0;
-	while ( col < nCols-1 && x >= xn+(xcd=cellWidth(col)) ) {
-	    xn += xcd;
-	    col++;
-	}
-	xCellOffs = col;
-	if ( testTableFlags(Tbl_snapToHGrid) ) {
-	    xCellDelta = 0;
-	    x = xn;
+	if ( cellH ) {
+		if ( y > maxYOffset() )
+			y = maxYOffset();
+		yCellOffs = y / cellH;
+		if ( !testTableFlags(Tbl_snapToVGrid) ) {
+			yCellDelta	= (short)(y % cellH);
+		} else {
+			y = yCellOffs*cellH;
+			yCellDelta	= 0;
+		}
 	} else {
-	    xCellDelta = (short)(x-xn);
+		int yn=0, yrd=0, row=0;
+		while ( row < nRows-1 && y >= yn+(yrd=cellHeight(row)) ) {
+			yn += yrd;
+			row++;
+		}
+		yCellOffs = row;
+		if ( testTableFlags(Tbl_snapToVGrid) ) {
+			yCellDelta = 0;
+			y = yn;
+		} else {
+			yCellDelta = (short)(y-yn);
+		}
 	}
-    }
-    if ( cellH ) {
-	if ( y > maxYOffset() )
-	    y = maxYOffset();
-	yCellOffs = y / cellH;
-	if ( !testTableFlags(Tbl_snapToVGrid) ) {
-	    yCellDelta	= (short)(y % cellH);
-	} else {
-	    y		= yCellOffs*cellH;
-	    yCellDelta	= 0;
+	int dx = (x - xOffs);
+	int dy = (y - yOffs);
+	xOffs = x;
+	yOffs_prev=yOffs;
+	yOffs = y;
+	if ( autoUpdate() && isVisible() )
+	{
+		scroll( dx, dy );
+		flag_scrolling=true;
 	}
-    } else {
-	int yn=0, yrd=0, row=0;
-	while ( row < nRows-1 && y >= yn+(yrd=cellHeight(row)) ) {
-	    yn += yrd;
-	    row++;
-	}
-	yCellOffs = row;
-	if ( testTableFlags(Tbl_snapToVGrid) ) {
-	    yCellDelta = 0;
-	    y = yn;
-	} else {
-	    yCellDelta = (short)(y-yn);
-	}
-    }
-    int dx = (x - xOffs);
-    int dy = (y - yOffs);
-    xOffs = x;
-    yOffs = y;
-    if ( autoUpdate() && isVisible() )
-	scroll( dx, dy );
-    if ( updateScrBars )
-	updateScrollBars( verValue | horValue );
+	if ( updateScrBars )
+		updateScrollBars( verValue | horValue );
 }
 
 
@@ -591,7 +596,7 @@ void QtTableView::setCellWidth( int cellWidth )
 
     updateScrollBars( horSteps | horRange );
     if ( autoUpdate() && isVisible() )
-	repaint();
+		repaint();
 
 }
 
@@ -632,6 +637,7 @@ int QtTableView::cellHeight( int )
 
 void QtTableView::setCellHeight( int cellHeight )
 {
+    
     if ( cellH == cellHeight )
 	return;
 #if defined(QT_CHECK_RANGE)
@@ -642,6 +648,8 @@ void QtTableView::setCellHeight( int cellHeight )
     }
 #endif
     cellH = (short)cellHeight;
+    QtTableView::cellheight = (short)cellHeight;
+    
     if ( autoUpdate() && isVisible() )
 	repaint();
     updateScrollBars( verSteps | verRange );
@@ -661,14 +669,14 @@ void QtTableView::setCellHeight( int cellHeight )
 
 int QtTableView::totalWidth()
 {
-    if ( cellW ) {
-	return cellW*nCols;
-    } else {
-	int tw = 0;
-	for( int i = 0 ; i < nCols ; i++ )
-	    tw += cellWidth( i );
-	return tw;
-    }
+	if ( cellW ) {
+		return cellW*nCols;
+	} else {
+		int tw = 0;
+		for( int i = 0 ; i < nCols ; i++ )
+			tw += cellWidth( i );
+		return tw;
+	}
 }
 
 /*
@@ -932,14 +940,15 @@ void QtTableView::clearTableFlags( uint f )
   \sa autoUpdate(), repaint()
 */
 
+// ?? 
 void QtTableView::setAutoUpdate( bool enable )
 {
     if ( isUpdatesEnabled() == enable )
-	return;
+        return;
     setUpdatesEnabled( enable );
     if ( enable ) {
-	showOrHideScrollBars();
-	updateScrollBars();
+        showOrHideScrollBars();
+        updateScrollBars();
     }
 }
 
@@ -953,17 +962,29 @@ void QtTableView::setAutoUpdate( bool enable )
   \sa isVisible()
 */
 
-void QtTableView::updateCell( int row, int col, bool erase )
+void drawXXX(QPainter *p,QRect r)
+{
+	p->fillRect(r ,QColor(255,0,0));
+}
+
+void QtTableView::updateCell( int row, int col, bool forced )
 {
     int xPos, yPos;
     if ( !colXPos( col, &xPos ) )
-	return;
+	    return;
     if ( !rowYPos( row, &yPos ) )
-	return;
-    QRect uR = QRect( xPos, yPos,
-		      cellW ? cellW : cellWidth(col),
-		      cellH ? cellH : cellHeight(row) );
-    repaint( uR.intersect(viewRect()), erase );
+	    return;
+
+	///forced=false; 
+    QRect uR = QRect( xPos, yPos,  
+                    cellW ? cellW : cellWidth(col),  cellH ? cellH : cellHeight(row) );
+                   // ???? cellWidth(col),  cellHeight(row) );
+ 	if(forced==false)
+		flag_update=true;
+	else 
+		flag_update=false;
+    repaint( uR.intersect(viewRect()), false );
+	flag_update=false;
 }
 
 
@@ -1071,21 +1092,21 @@ bool QtTableView::colIsVisible( int col ) const
 
 void QtTableView::coverCornerSquare( bool enable )
 {
-    coveringCornerSquare = enable;
-    if ( !cornerSquare && enable ) {
-	cornerSquare = new QCornerSquare( this );
-	Q_CHECK_PTR( cornerSquare );
-	cornerSquare->setGeometry( maxViewX() + frameWidth() + 1,
-				   maxViewY() + frameWidth() + 1,
-                                   VSBEXT,
-                                 HSBEXT);
-    }
-    if ( autoUpdate() && cornerSquare ) {
-	if ( enable )
-	    cornerSquare->show();
-	else
-	    cornerSquare->hide();
-    }
+	coveringCornerSquare = enable;
+	if ( !cornerSquare && enable ) {
+		cornerSquare = new QCornerSquare( this );
+		Q_CHECK_PTR( cornerSquare );
+		cornerSquare->setGeometry( maxViewX() + frameWidth() + 1,
+				maxViewY() + frameWidth() + 1,
+				VSBEXT,
+				HSBEXT);
+	}
+	if ( autoUpdate() && cornerSquare ) {
+		if ( enable )
+			cornerSquare->show();
+		else
+			cornerSquare->hide();
+	}
 }
 
 
@@ -1271,52 +1292,51 @@ void QtTableView::setupPainter( QPainter * )
 
 void QtTableView::paintEvent( QPaintEvent *e )
 {
-    QRect updateR = e->rect();			// update rectangle
-    if ( sbDirty ) {
-	bool e = eraseInPaint;
-	updateScrollBars();
-	eraseInPaint = e;
-    }
+	QRect updateR = e->rect();			// update rectangle
+	if ( sbDirty ) {
+		bool e = eraseInPaint;
+		updateScrollBars();
+		eraseInPaint = e;
+	}
 
-    QPainter paint( this );
+	QPainter paint( this );
 
-    if ( !contentsRect().contains( updateR, TRUE  ) ) {// update frame ?
-	drawFrame( &paint );
-	if ( updateR.left() < frameWidth() ) 		//###
-	    updateR.setLeft( frameWidth() );
-	if ( updateR.top() < frameWidth() )
-	    updateR.setTop( frameWidth() );
-    }
+	if ( !contentsRect().contains( updateR, TRUE  ) ) {// update frame ?
+		drawFrame( &paint );
+		if ( updateR.left() < frameWidth() ) 		//###
+			updateR.setLeft( frameWidth() );
+		if ( updateR.top() < frameWidth() )
+			updateR.setTop( frameWidth() );
+	}
 
-    int maxWX = maxViewX();
-    int maxWY = maxViewY();
-    if ( updateR.right() > maxWX )
-	updateR.setRight( maxWX );
-    if ( updateR.bottom() > maxWY )
-	updateR.setBottom( maxWY );
-
-    setupPainter( &paint );			// prepare for painting table
+	int maxWX = maxViewX();
+	int maxWY = maxViewY();
+	if ( updateR.right() > maxWX )
+		updateR.setRight( maxWX );
+	if ( updateR.bottom() > maxWY )
+		updateR.setBottom( maxWY );
 
+    setupPainter( &paint );			// prepare for painting table    
+    paint.setBackgroundColor(backColor);
+    
     int firstRow = findRow( updateR.y() );
     int firstCol = findCol( updateR.x() );
     int	 xStart, yStart;
-    
-    //if ( !rowYPos( firstRow, &yStart ) || !colXPos( firstCol, &xStart )  ) {
-    if ( !rowYPos( firstRow, &yStart ) ) {
-	paint.eraseRect( updateR ); // erase area outside cells but in view
-	return;
-    }
-  
-    if ( !colXPos( firstCol, &xStart ) ) {
-	//paint.eraseRect( updateR ); // erase area outside cells but in view
-	firstCol=nCols-1;
-    	colXPos( firstCol, &xStart );
-	//return;
-    } 
+   
+	if ( !colXPos( firstCol, &xStart ) ) {
+		eraseRight(&paint,updateR);
+	//	paint.eraseRect( updateR ); // erase area outside cells but in view 
+		return;
+	} 
+
+	//if ( !rowYPos( firstRow, &yStart ) || !colXPos( firstCol, &xStart )  ) {
+	if ( !rowYPos( firstRow, &yStart ) ) {	    
+		paint.eraseRect( updateR ); // erase area outside cells but in view 
+		return;
+	}
 
 
-
-    int	  maxX	= updateR.right();
+	int	  maxX	= updateR.right();
     int	  maxY	= updateR.bottom();
     int	  row	= firstRow;
     int	  col;
@@ -1327,66 +1347,45 @@ void QtTableView::paintEvent( QPaintEvent *e )
     QRect winR = viewRect();
     QRect cellR;
     QRect cellUR;
-#ifndef QT_NO_TRANSFORMATIONS
-    QWMatrix matrix;
-#endif
 
-    while ( yPos <= maxY && row < nRows ) {
-	nextY = yPos + (cellH ? cellH : cellHeight( row ));
-	if ( testTableFlags( Tbl_cutCellsV ) && nextY > ( maxWY + 1 ) )
-	    break;
-	
-	col  = firstCol;
-	xPos = xStart;
-	
-	while ( xPos <= maxX && col < nCols ) {
-	    nextX = xPos + (cellW ? cellW : cellWidth( col ));
-	    if ( testTableFlags( Tbl_cutCellsH ) && nextX > ( maxWX + 1 ) )
-		break;
+	while ( yPos <= maxY && row < nRows ) {
+		nextY = yPos + (cellH ? cellH : cellHeight( row ));
+		if ( testTableFlags( Tbl_cutCellsV ) && nextY > ( maxWY + 1 ) )
+			break;
 
-	    cellR.setRect( xPos, yPos, cellW ? cellW : cellWidth(col),
-				       cellH ? cellH : cellHeight(row) );
-	    cellUR = cellR.intersect( updateR );
-	    //if ( cellUR.isValid() ) {
-	    if (1) {
-	    	//if (col==nCols-1) printf("kkkk3333xxxxxxxxx111111111111\n");
-		cellUpdateR = cellUR;
-		cellUpdateR.moveBy( -xPos, -yPos ); // cell coordinates
-		//DEL if ( eraseInPaint )    paint.eraseRect( cellUR );
+		col  = firstCol;
+		xPos = xStart;
 
-#ifndef QT_NO_TRANSFORMATIONS
-		matrix.translate( xPos, yPos );
-		paint.setWorldMatrix( matrix );
-		if ( testTableFlags(Tbl_clipCellPainting) ||
-		     frameWidth() > 0 && !winR.contains( cellR ) ) { //##arnt
-		    paint.setClipRect( cellUR );
-		    paintCell( &paint, row, col );
-		    paint.setClipping( FALSE );
-		} else {
-		    paintCell( &paint, row, col );
+		while ( xPos <= maxX && col < nCols ) {
+			nextX = xPos + (cellW ? cellW : cellWidth( col ));
+			if ( testTableFlags( Tbl_cutCellsH ) && nextX > ( maxWX + 1 ) )
+				break;
+
+			cellR.setRect( xPos, yPos, cellW ? cellW : cellWidth(col),
+					cellH ? cellH : cellHeight(row) );
+			cellUR = cellR.intersect( updateR );
+			if ( cellUR.isValid() ) {
+				cellUpdateR = cellUR;
+				cellUpdateR.moveBy( -xPos, -yPos ); // cell coordinates
+				
+				paint.translate( xPos, yPos );
+				if ( testTableFlags(Tbl_clipCellPainting) ||
+						frameWidth() > 0 && !winR.contains( cellR ) ) { //##arnt
+					paint.setClipRect( cellUR );
+					paintCell( &paint, row, col ,flag_update);
+					paint.setClipping( FALSE );
+				} else {
+					paintCell( &paint, row, col ,flag_update);
+				}
+				paint.translate( -xPos, -yPos );
+			}
+			col++;
+			xPos = nextX;
 		}
-		matrix.reset();
-		paint.setWorldMatrix( matrix );
-#else
-		paint.translate( xPos, yPos );
-		if ( testTableFlags(Tbl_clipCellPainting) ||
-		     frameWidth() > 0 && !winR.contains( cellR ) ) { //##arnt
-		    paint.setClipRect( cellUR );
-		    paintCell( &paint, row, col );
-		    paint.setClipping( FALSE );
-		} else {
-		    paintCell( &paint, row, col );
+
+		row++;
+		yPos = nextY;
 		}
-		paint.translate( -xPos, -yPos );
-#endif
-	    }
-	    col++;
-	    xPos = nextX;
-	}
-	
-	row++;
-	yPos = nextY;
-    }
 
     // while painting we have to erase any areas in the view that
     // are not covered by cells but are covered by the paint event
@@ -1403,26 +1402,20 @@ void QtTableView::paintEvent( QPaintEvent *e )
     QRect viewR = viewRect();
     const QColorGroup g = colorGroup();
 
-    if ( xPos <= maxX ) {
-	QRect r = viewR;
-	r.setLeft( xPos );
-	r.setBottom( yPos<maxY?yPos:maxY );
-	/*
-	if ( inherits( "QMultiLineEdit" ) )
-	    paint.fillRect( r.intersect( updateR ), g.base() );
-	else
-	    paint.eraseRect( r.intersect( updateR ) );
-	  */
-	//paint.fillRect( r.intersect( updateR ), g.base() );
-    }
-    if ( yPos <= maxY ) {
-	QRect r = viewR;
-	r.setTop( yPos );
-	if ( inherits( "QMultiLineEdit" ) )
-	    paint.fillRect( r.intersect( updateR ), g.base() );
-	else
-	    paint.eraseRect( r.intersect( updateR ) );
-    }
+	if ( xPos <= maxX ) {
+		QRect r = viewR;
+		r.setLeft( xPos );
+		r.setBottom( yPos<maxY?yPos:maxY );
+		paint.eraseRect( r.intersect( updateR ) );
+		QRect ir=r.intersect( updateR );
+		eraseRight(&paint, ir);
+	}
+	if ( yPos <= maxY ) {
+		QRect r = viewR;
+		r.setTop( yPos );
+		paint.eraseRect( r.intersect( updateR ) );
+	}
+	flag_scrolling=false;
 }
 
 /*\reimp
@@ -1445,7 +1438,11 @@ void QtTableView::resizeEvent( QResizeEvent * )
 
 void QtTableView::updateView()
 {
+	flag_update=true;
+//	flag_update=false;
+	//printf("updateView()\n");
     repaint( viewRect() );
+	flag_update=false;
 }
 
 /*
@@ -1459,10 +1456,8 @@ QScrollBar *QtTableView::verticalScrollBar() const
     QtTableView *that = (QtTableView*)this; // semantic const
     if ( !vScrollBar ) {
 	QScrollBar *sb = new QScrollBar( QScrollBar::Vertical, that );
-#ifndef QT_NO_CURSOR
-	sb->setCursor( arrowCursor );
-#endif
-        sb->resize( sb->sizeHint() ); // height is irrelevant
+
+    sb->resize( sb->sizeHint() ); // height is irrelevant
 	Q_CHECK_PTR(sb);
 	sb->setTracking( FALSE );
 	sb->setFocusPolicy( NoFocus );
@@ -1490,9 +1485,6 @@ QScrollBar *QtTableView::horizontalScrollBar() const
     QtTableView *that = (QtTableView*)this; // semantic const
     if ( !hScrollBar ) {
 	QScrollBar *sb = new QScrollBar( QScrollBar::Horizontal, that );
-#ifndef QT_NO_CURSOR
-	sb->setCursor( arrowCursor );
-#endif
 	sb->resize( sb->sizeHint() ); // width is irrelevant
 	sb->setFocusPolicy( NoFocus );
 	Q_CHECK_PTR(sb);
@@ -1713,13 +1705,13 @@ int QtTableView::findRow( int yPos ) const
 
 int QtTableView::findCol( int xPos ) const
 {
-    int cellMaxX;
-    int col = findRawCol( xPos, &cellMaxX );
-    if ( testTableFlags(Tbl_cutCellsH) && cellMaxX > maxViewX() )
-	col = - 1;				//  cell cut by right margin
-    if ( col >= nCols )
-	col = -1;
-    return col;
+	int cellMaxX;
+	int col = findRawCol( xPos, &cellMaxX );
+	if ( testTableFlags(Tbl_cutCellsH) && cellMaxX > maxViewX() )
+		col = - 1;				//  cell cut by right margin
+	if ( col >= nCols )
+		col = -1;
+	return col;
 }
 
 
@@ -1733,33 +1725,34 @@ int QtTableView::findCol( int xPos ) const
   \sa colXPos(), findRow()
 */
 
+// return : FALSE means out of bound
 bool QtTableView::rowYPos( int row, int *yPos ) const
 {
-    int y;
-    if ( row >= yCellOffs ) {
-	if ( cellH ) {
-	    int lastVisible = lastRowVisible();
-	    if ( row > lastVisible || lastVisible == -1 )
-		return FALSE;
-	    y = (row - yCellOffs)*cellH + minViewY() - yCellDelta;
-	} else {
-	    //##arnt3
-	    y = minViewY() - yCellDelta;	// y of leftmost cell in view
-	    int r = yCellOffs;
-	    QtTableView *tw = (QtTableView *)this;
-	    int maxY = maxViewY();
-	    while ( r < row && y <= maxY )
-		y += tw->cellHeight( r++ );
-	    if ( y > maxY )
-		return FALSE;
+	int y;
+	if ( row >= yCellOffs ) {
+		if ( cellH ) {
+			int lastVisible = lastRowVisible();
+			if ( row > lastVisible || lastVisible == -1 )
+				return FALSE;
+			y = (row - yCellOffs)*cellH + minViewY() - yCellDelta;
+		} else {
+			//##arnt3
+			y = minViewY() - yCellDelta;	// y of leftmost cell in view
+			int r = yCellOffs;
+			QtTableView *tw = (QtTableView *)this;
+			int maxY = maxViewY();
+			while ( r < row && y <= maxY )
+				y += tw->cellHeight( r++ );
+			if ( y > maxY )
+				return FALSE;
 
+		}
+	} else {
+		return FALSE;
 	}
-    } else {
-	return FALSE;
-    }
-    if ( yPos )
-	*yPos = y;
-    return TRUE;
+	if ( yPos )
+		*yPos = y;
+	return TRUE;
 }
 
 
@@ -1777,27 +1770,27 @@ bool QtTableView::colXPos( int col, int *xPos ) const
 {
     int x;
     if ( col >= xCellOffs ) {
-	if ( cellW ) {
-	    int lastVisible = lastColVisible();
-	    if ( col > lastVisible || lastVisible == -1 )
-		return FALSE;
-	    x = (col - xCellOffs)*cellW + minViewX() - xCellDelta;
-	} else {
-	    //##arnt3
-	    x = minViewX() - xCellDelta;	// x of uppermost cell in view
-	    int c = xCellOffs;
-	    QtTableView *tw = (QtTableView *)this;
-	    int maxX = maxViewX();
-	    while ( c < col && x <= maxX )
-		x += tw->cellWidth( c++ );
-	    if ( x > maxX )
-		return FALSE;
-	}
+        if ( cellW ) {
+            int lastVisible = lastColVisible();
+            if ( col > lastVisible || lastVisible == -1 )
+                return FALSE;
+            x = (col - xCellOffs)*cellW + minViewX() - xCellDelta;
+        } else {
+            //##arnt3
+            x = minViewX() - xCellDelta;	// x of uppermost cell in view
+            int c = xCellOffs;
+            QtTableView *tw = (QtTableView *)this;
+            int maxX = maxViewX();
+            while ( c < col && x <= maxX )
+                x += tw->cellWidth( c++ );
+            if ( x > maxX )
+                return FALSE;
+        }
     } else {
-	return FALSE;
+        return FALSE;
     }
     if ( xPos )
-	*xPos = x;
+        *xPos = x;
     return TRUE;
 }
 
@@ -1859,8 +1852,7 @@ int QtTableView::minViewY() const
 int QtTableView::maxViewX() const
 {
     return width() - 1 - frameWidth()
-        - (tFlags & Tbl_vScrollBar ? VSBEXT
-           : 0);
+        - (tFlags & Tbl_vScrollBar ? VSBEXT : 0);
 }
 
 
@@ -1874,9 +1866,7 @@ int QtTableView::maxViewX() const
 
 int QtTableView::maxViewY() const
 {
-    return height() - 1 - frameWidth()
-        - (tFlags & Tbl_hScrollBar ? HSBEXT
-           : 0);
+    return height() - 1 - frameWidth() - (tFlags & Tbl_hScrollBar ? HSBEXT : 0);
 }
 
 
@@ -1963,127 +1953,123 @@ void QtTableView::doAutoScrollBars()
 
 /*
   \fn void QtTableView::updateScrollBars()
-
   Updates the scroll bars' contents and presence to match the table's
   state.  Generally, you should not need to call this.
-
   \sa setTableFlags()
 */
 
 /*
   Updates the scroll bars' contents and presence to match the table's
   state \c or \a f.
-
   \sa setTableFlags()
 */
 
 void QtTableView::updateScrollBars( uint f )
 {
-    sbDirty = sbDirty | f;
-    if ( inSbUpdate )
-	return;
-    inSbUpdate = TRUE;
+	sbDirty = sbDirty | f;
+	if ( inSbUpdate )
+		return;
+	inSbUpdate = TRUE;
 
-    if ( testTableFlags(Tbl_autoHScrollBar) && (sbDirty & horRange) ||
-	 testTableFlags(Tbl_autoVScrollBar) && (sbDirty & verRange) )
-					// if range change and auto
-	doAutoScrollBars();		// turn scroll bars on/off if needed
+	if ( testTableFlags(Tbl_autoHScrollBar) && (sbDirty & horRange) ||
+			testTableFlags(Tbl_autoVScrollBar) && (sbDirty & verRange) )
+		// if range change and auto
+		doAutoScrollBars();		// turn scroll bars on/off if needed
 
-    if ( !autoUpdate() ) {
-	inSbUpdate = FALSE;
-	return;
-    }
-    if ( yOffset() > 0 && testTableFlags( Tbl_autoVScrollBar ) &&
-	 !testTableFlags( Tbl_vScrollBar ) ) {
-	setYOffset( 0 );
-    }
-    if ( xOffset() > 0 && testTableFlags( Tbl_autoHScrollBar ) &&
-	 !testTableFlags( Tbl_hScrollBar ) ) {
-	setXOffset( 0 );
-    }
-    if ( !isVisible() ) {
-	inSbUpdate = FALSE;
-	return;
-    }
-
-    if ( testTableFlags(Tbl_hScrollBar) && (sbDirty & horMask) != 0 ) {
-	if ( sbDirty & horGeometry )
-	    hScrollBar->setGeometry( 0,height() - HSBEXT,
-                                     viewWidth() + frameWidth()*2,
-                                   HSBEXT);
-
-	if ( sbDirty & horSteps ) {
-	    if ( cellW )
-		hScrollBar->setSteps( QMIN(cellW,viewWidth()/2), viewWidth() );
-	    else
-		hScrollBar->setSteps( 16, viewWidth() );
+	if ( !autoUpdate() ) {
+		inSbUpdate = FALSE;
+		return;
+	}
+	if ( yOffset() > 0 && testTableFlags( Tbl_autoVScrollBar ) &&
+			!testTableFlags( Tbl_vScrollBar ) ) {
+		setYOffset( 0 );
+	}
+	if ( xOffset() > 0 && testTableFlags( Tbl_autoHScrollBar ) &&
+			!testTableFlags( Tbl_hScrollBar ) ) {
+		setXOffset( 0 );
+	}
+	if ( !isVisible() ) {
+		inSbUpdate = FALSE;
+		return;
 	}
 
-	if ( sbDirty & horRange )
-	    hScrollBar->setRange( 0, maxXOffset() );
+	if ( testTableFlags(Tbl_hScrollBar) && (sbDirty & horMask) != 0 ) {
+		if ( sbDirty & horGeometry )
+			hScrollBar->setGeometry( 0,height() - HSBEXT,
+					viewWidth() + frameWidth()*2,
+					HSBEXT);
 
-	if ( sbDirty & horValue )
-	    hScrollBar->setValue( xOffs );
+		if ( sbDirty & horSteps ) {
+			if ( cellW )
+				hScrollBar->setSteps( QMIN(cellW,viewWidth()/2), viewWidth() );
+			else
+				hScrollBar->setSteps( 16, viewWidth() );
+		}
 
-			// show scrollbar only when it has a sane geometry
-	if ( !hScrollBar->isVisible() )
-	    hScrollBar->show();
-    }
+		if ( sbDirty & horRange )
+			hScrollBar->setRange( 0, maxXOffset() );
 
-    if ( testTableFlags(Tbl_vScrollBar) && (sbDirty & verMask) != 0 ) {
-	if ( sbDirty & verGeometry )
-	    vScrollBar->setGeometry( width() - VSBEXT, 0,
-                                     VSBEXT,
-                                     viewHeight() + frameWidth()*2 );
+		if ( sbDirty & horValue )
+			hScrollBar->setValue( xOffs );
 
-	if ( sbDirty & verSteps ) {
-	    if ( cellH )
-		vScrollBar->setSteps( QMIN(cellH,viewHeight()/2), viewHeight() );
-	    else
-		vScrollBar->setSteps( 16, viewHeight() );  // fttb! ###
+		// show scrollbar only when it has a sane geometry
+		if ( !hScrollBar->isVisible() )
+			hScrollBar->show();
 	}
 
-	if ( sbDirty & verRange )
-	    vScrollBar->setRange( 0, maxYOffset() );
+	if ( testTableFlags(Tbl_vScrollBar) && (sbDirty & verMask) != 0 ) {
+		if ( sbDirty & verGeometry )
+			vScrollBar->setGeometry( width() - VSBEXT, 0,
+					VSBEXT,
+					viewHeight() + frameWidth()*2 );
 
-	if ( sbDirty & verValue )
-	    vScrollBar->setValue( yOffs );
+		if ( sbDirty & verSteps ) {
+			if ( cellH )
+				vScrollBar->setSteps( QMIN(cellH,viewHeight()/2), viewHeight() );
+			else
+				vScrollBar->setSteps( 16, viewHeight() );  // fttb! ###
+		}
 
-			// show scrollbar only when it has a sane geometry
-	if ( !vScrollBar->isVisible() )
-	    vScrollBar->show();
-    }
-    if ( coveringCornerSquare &&
-	 ( (sbDirty & verGeometry ) || (sbDirty & horGeometry)) )
-	cornerSquare->move( maxViewX() + frameWidth() + 1,
-			    maxViewY() + frameWidth() + 1 );
+		if ( sbDirty & verRange )
+			vScrollBar->setRange( 0, maxYOffset() );
 
-    sbDirty = 0;
-    inSbUpdate = FALSE;
+		if ( sbDirty & verValue )
+			vScrollBar->setValue( yOffs );
+
+		// show scrollbar only when it has a sane geometry
+		if ( !vScrollBar->isVisible() )
+			vScrollBar->show();
+	}
+	if ( coveringCornerSquare &&
+			( (sbDirty & verGeometry ) || (sbDirty & horGeometry)) )
+		cornerSquare->move( maxViewX() + frameWidth() + 1,
+				maxViewY() + frameWidth() + 1 );
+
+	sbDirty = 0;
+	inSbUpdate = FALSE;
 }
 
 
 void QtTableView::updateFrameSize()
 {
-    int rw = width()  - ( testTableFlags(Tbl_vScrollBar) ?
-                          VSBEXT : 0 );
-    int rh = height() - ( testTableFlags(Tbl_hScrollBar) ?
-                          HSBEXT : 0 );
-    if ( rw < 0 )
-	rw = 0;
-    if ( rh < 0 )
-	rh = 0;
+	int rw = width()  - ( testTableFlags(Tbl_vScrollBar) ?	VSBEXT : 0 );
+	int rh = height() - ( testTableFlags(Tbl_hScrollBar) ?	HSBEXT : 0 );
 
-    if ( autoUpdate() ) {
-        int fh = frameRect().height();
-	int fw = frameRect().width();
-	setFrameRect( QRect(0,0,rw,rh) );
+	if ( rw < 0 )
+		rw = 0;
+	if ( rh < 0 )
+		rh = 0;
 
-	if ( rw != fw )
-	    update( QMIN(fw,rw) - frameWidth() - 2, 0, frameWidth()+4, rh );
-	if ( rh != fh )
-	    update( 0, QMIN(fh,rh) - frameWidth() - 2, rw, frameWidth()+4 );
-    }
+	if ( autoUpdate() ) {
+		int fh = frameRect().height();
+		int fw = frameRect().width();
+		setFrameRect( QRect(0,0,rw,rh) );
+
+		if ( rw != fw )
+			update( QMIN(fw,rw) - frameWidth() - 2, 0, frameWidth()+4, rh );
+		if ( rh != fh )
+			update( 0, QMIN(fh,rh) - frameWidth() - 2, rw, frameWidth()+4 );
+	}
 }
 
 
@@ -2098,38 +2084,38 @@ void QtTableView::updateFrameSize()
 
 int QtTableView::maxXOffset()
 {
-    int tw = totalWidth();
-    int maxOffs;
-    if ( testTableFlags(Tbl_scrollLastHCell) ) {
-	if ( nCols != 1)
-	    maxOffs =  tw - ( cellW ? cellW : cellWidth( nCols - 1 ) );
-	else
-	    maxOffs = tw - viewWidth();
-    } else {
-	if ( testTableFlags(Tbl_snapToHGrid) ) {
-	    if ( cellW ) {
-		maxOffs =  tw - (viewWidth()/cellW)*cellW;
-	    } else {
-		int goal = tw - viewWidth();
-		int pos = tw;
-		int nextCol = nCols - 1;
-		int nextCellWidth = cellWidth( nextCol );
-		while( nextCol > 0 && pos > goal + nextCellWidth ) {
-		    pos -= nextCellWidth;
-		    nextCellWidth = cellWidth( --nextCol );
-		}
-		if ( goal + nextCellWidth == pos )
-		    maxOffs = goal;
-		 else if ( goal < pos )
-		   maxOffs = pos;
-		 else
-		   maxOffs = 0;
-	    }
+	int tw = totalWidth();
+	int maxOffs;
+	if ( testTableFlags(Tbl_scrollLastHCell) ) {
+		if ( nCols != 1)
+			maxOffs =  tw - ( cellW ? cellW : cellWidth( nCols - 1 ) );
+		else
+			maxOffs = tw - viewWidth();
 	} else {
-	    maxOffs = tw - viewWidth();
+		if ( testTableFlags(Tbl_snapToHGrid) ) {
+			if ( cellW ) {
+				maxOffs =  tw - (viewWidth()/cellW)*cellW;
+			} else {
+				int goal = tw - viewWidth();
+				int pos = tw;
+				int nextCol = nCols - 1;
+				int nextCellWidth = cellWidth( nextCol );
+				while( nextCol > 0 && pos > goal + nextCellWidth ) {
+					pos -= nextCellWidth;
+					nextCellWidth = cellWidth( --nextCol );
+				}
+				if ( goal + nextCellWidth == pos )
+					maxOffs = goal;
+				else if ( goal < pos )
+					maxOffs = pos;
+				else
+					maxOffs = 0;
+			}
+		} else {
+			maxOffs = tw - viewWidth();
+		}
 	}
-    }
-    return maxOffs > 0 ? maxOffs : 0;
+	return maxOffs > 0 ? maxOffs : 0;
 }
 
 
@@ -2144,38 +2130,38 @@ int QtTableView::maxXOffset()
 
 int QtTableView::maxYOffset()
 {
-    int th = totalHeight();
-    int maxOffs;
-    if ( testTableFlags(Tbl_scrollLastVCell) ) {
-	if ( nRows != 1)
-	    maxOffs =  th - ( cellH ? cellH : cellHeight( nRows - 1 ) );
-	else
-	    maxOffs = th - viewHeight();
-    } else {
-	if ( testTableFlags(Tbl_snapToVGrid) ) {
-	    if ( cellH ) {
-		maxOffs =  th - (viewHeight()/cellH)*cellH;
-	    } else {
-		int goal = th - viewHeight();
-		int pos = th;
-		int nextRow = nRows - 1;
-		int nextCellHeight = cellHeight( nextRow );
-		while( nextRow > 0 && pos > goal + nextCellHeight ) {
-		    pos -= nextCellHeight;
-		    nextCellHeight = cellHeight( --nextRow );
-		}
-		if ( goal + nextCellHeight == pos )
-		    maxOffs = goal;
-		 else if ( goal < pos )
-		   maxOffs = pos;
-		 else
-		   maxOffs = 0;
-	    }
+	int th = totalHeight();
+	int maxOffs;
+	if ( testTableFlags(Tbl_scrollLastVCell) ) {
+		if ( nRows != 1)
+			maxOffs =  th - ( cellH ? cellH : cellHeight( nRows - 1 ) );
+		else
+			maxOffs = th - viewHeight();
 	} else {
-	    maxOffs = th - viewHeight();
+		if ( testTableFlags(Tbl_snapToVGrid) ) {
+			if ( cellH ) {
+				maxOffs =  th - (viewHeight()/cellH)*cellH;
+			} else {
+				int goal = th - viewHeight();
+				int pos = th;
+				int nextRow = nRows - 1;
+				int nextCellHeight = cellHeight( nextRow );
+				while( nextRow > 0 && pos > goal + nextCellHeight ) {
+					pos -= nextCellHeight;
+					nextCellHeight = cellHeight( --nextRow );
+				}
+				if ( goal + nextCellHeight == pos )
+					maxOffs = goal;
+				else if ( goal < pos )
+					maxOffs = pos;
+				else
+					maxOffs = 0;
+			}
+		} else {
+			maxOffs = th - viewHeight();
+		}
 	}
-    }
-    return maxOffs > 0 ? maxOffs : 0;
+	return maxOffs > 0 ? maxOffs : 0;
 }
 
 
@@ -2191,17 +2177,17 @@ int QtTableView::maxYOffset()
 
 int QtTableView::maxColOffset()
 {
-    int mx = maxXOffset();
-    if ( cellW )
-	return mx/cellW;
-    else {
-	int xcd=0, col=0;
-	while ( col < nCols && mx > (xcd=cellWidth(col)) ) {
-	    mx -= xcd;
-	    col++;
+	int mx = maxXOffset();
+	if ( cellW )
+		return mx/cellW;
+	else {
+		int xcd=0, col=0;
+		while ( col < nCols && mx > (xcd=cellWidth(col)) ) {
+			mx -= xcd;
+			col++;
+		}
+		return col;
 	}
-	return col;
-    }
 }
 
 
@@ -2233,48 +2219,46 @@ int QtTableView::maxRowOffset()
 
 void QtTableView::showOrHideScrollBars()
 {
-    if ( !autoUpdate() )
-	return;
-    if ( vScrollBar ) {
-	if ( testTableFlags(Tbl_vScrollBar) ) {
-	    if ( !vScrollBar->isVisible() )
-		sbDirty = sbDirty | verMask;
-	} else {
-	    if ( vScrollBar->isVisible() )
-	       vScrollBar->hide();
+	if ( !autoUpdate() )
+		return;
+	if ( vScrollBar ) {
+		if ( testTableFlags(Tbl_vScrollBar) ) {
+			if ( !vScrollBar->isVisible() )
+				sbDirty = sbDirty | verMask;
+		} else {
+			if ( vScrollBar->isVisible() )
+				vScrollBar->hide();
+		}
 	}
-    }
-    if ( hScrollBar ) {
-	if ( testTableFlags(Tbl_hScrollBar) ) {
-	    if ( !hScrollBar->isVisible() )
-		sbDirty = sbDirty | horMask;
-	} else {
-	    if ( hScrollBar->isVisible() )
-		hScrollBar->hide();
+	if ( hScrollBar ) {
+		if ( testTableFlags(Tbl_hScrollBar) ) {
+			if ( !hScrollBar->isVisible() )
+				sbDirty = sbDirty | horMask;
+		} else {
+			if ( hScrollBar->isVisible() )
+				hScrollBar->hide();
+		}
 	}
-    }
-    if ( cornerSquare ) {
-	if ( testTableFlags(Tbl_hScrollBar) &&
-	     testTableFlags(Tbl_vScrollBar) ) {
-	    if ( !cornerSquare->isVisible() )
-		cornerSquare->show();
-	} else {
-	    if ( cornerSquare->isVisible() )
-		cornerSquare->hide();
+	if ( cornerSquare ) {
+		if ( testTableFlags(Tbl_hScrollBar) &&
+				testTableFlags(Tbl_vScrollBar) ) {
+			if ( !cornerSquare->isVisible() )
+				cornerSquare->show();
+		} else {
+			if ( cornerSquare->isVisible() )
+				cornerSquare->hide();
+		}
 	}
-    }
 }
 
 
 /*
   Updates the scroll bars and internal state.
-
+  
   Call this function when the table view's total size is changed;
   typically because the result of cellHeight() or cellWidth() have changed.
-
   This function does not repaint the widget.
 */
-
 void QtTableView::updateTableSize()
 {
     bool updateOn = autoUpdate();
@@ -2284,10 +2268,9 @@ void QtTableView::updateTableSize()
     setOffset(xofs,yOffset(),FALSE); //to calculate internal state correctly
     setAutoUpdate(updateOn);
 
-    updateScrollBars( horSteps |  horRange |
-		      verSteps |  verRange );
+    updateScrollBars( horSteps |  horRange | verSteps |  verRange );
     showOrHideScrollBars();
 }
 
 
-#endif
+
