@@ -9,7 +9,7 @@
 #include <stdlib.h>
 
 #ifndef NDEBUG
-#define CHECK_INDICES		// Check invalid vector indices (the default)
+//#define CHECK_INDICES		// Check invalid vector indices (the default)
 #endif
 
 template<class T>
@@ -30,6 +30,7 @@ public:
     void add(T x);
     void insert(int index,T val);     
     void remove(int index);
+    void Delete(int index);
     void clear();
     void purge();		// like clear() but deletes all contents
 
@@ -73,9 +74,9 @@ template<class T>
 inline void Svec<T>::set(int i, T val)
 {
     if(i < 0 || i >= used)
-	setextend(i, val);
+		setextend(i, val);
     else
-	vect[i] = val;
+		vect[i] = val;
 }
 
 template<class T>
@@ -86,12 +87,29 @@ inline void Svec<T>::add(T x)
     vect[used - 1] = x;
 }
 
+/*
 template<class T>
-inline void Svec<T>::clear() { used = 0; }
+inline void Svec<T>::add(T x)
+{
+    if(++used > alloced)
+    {
+
+    }
+
+    vect[used - 1] = x;
+}
+*/
+
+template<class T>
+inline void Svec<T>::clear() 
+{ 
+    used = 0; 
+}
 
 template<class T>
 inline void Svec<T>::grow()
 {
+//	printf("size=%d\n",sizeof(T));
     vect = (T *)realloc(vect, (alloced *= 2) * sizeof(T));
 }
 
