@@ -18,7 +18,7 @@
 
 class Ttystr {
 public:
-    static QString name(int devnr);
+    static QString name(dev_t devnr);
 
 #ifdef SOLARIS
 private:
