@@ -1,7 +1,7 @@
 // uidstr.h
 //
 // This program is free software. See the file COPYING for details.
-// Author: Mattias Engdegrd, 1997, 1998
+// Author: Mattias Engdegrd, 1997-1999
 
 #ifndef UIDSTR_H
 #define UIDSTR_H
@@ -11,11 +11,12 @@
 
 class Uidstr {
 public:
-    static QString name(int uid);
+    static QString userName(int uid);
+    static QString groupName(int gid);
 
 private:
-    static QIntDict<char> dict;
-    
+    static QIntDict<char> udict;
+    static QIntDict<char> gdict;
 };
 
 #endif	// UIDSTR_H
