@@ -11,13 +11,13 @@
 
 class Wchan {
 public:
-    static QString name(unsigned addr);
+    static QString name(unsigned long addr);
 
 #ifdef LINUX
 private:
     static bool open_sysmap();
     static bool try_sysmap(const char *path);
-    static char *find_sym(unsigned addr);
+    static char *find_sym(unsigned long addr);
     static inline int beginning_of_line(int ofs);
 
     static QIntDict<char> dict;
